#region licence/info

//////project name
//vvvv plugin template

//////description
//basic vvvv node plugin template.
//Copy this an rename it, to write your own plugin node.

//////licence
//GNU Lesser General Public License (LGPL)
//english: http://www.gnu.org/licenses/lgpl.html
//german: http://www.gnu.de/lgpl-ger.html

//////language/ide
//C# sharpdevelop 

//////dependencies
//VVVV.PluginInterfaces.V1;
//VVVV.Utils.VColor;
//VVVV.Utils.VMath;

//////initial author
//vvvv group

#endregion licence/info

//use what you need
using System;
using System.IO;
using System.Drawing;
using System.Threading;
using System.Text;
using VVVV.PluginInterfaces.V1;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;
using InTheHand.Net;
using InTheHand.Net.Sockets;
using InTheHand.Net.Bluetooth;

//the vvvv node namespace
namespace VVVV.Nodes
{
	
	//class definition
	public class PluginTemplate: IPlugin, IDisposable
    {	          	
    	#region field declaration
    	
    	//the host (mandatory)
    	private IPluginHost FHost; 
    	// Track whether Dispose has been called.
   		private bool FDisposed = false;

   		//input pin declaration
    	private IStringIn dstFPTHStrInp;
    	private IValueIn listenValInp;
    	
    	//output pin declaration
    	private IStringOut msgStrOut;
    	private IValueOut receivedValOut;
    	private IStringOut fileStrOut;
    	private IStringOut macStrOut;

    	private Thread thread;
    	private ObexListener listener;
    	
    	private bool running = true;
    	private string path = "c:\\";
    	
    	private bool msg = false;
    	private bool received = false;
    	private string filename = "";
    	private string remoteMac = "";
    	
    	#endregion field declaration
       
    	#region constructor/destructor
    	
        public PluginTemplate()
        {
			if (BluetoothRadio.IsSupported) {
				if (BluetoothRadio.PrimaryRadio.Mode == RadioMode.PowerOff) {
					BluetoothRadio.PrimaryRadio.Mode = RadioMode.Discoverable;
				}
        	}
		}
        
        // Implementing IDisposable's Dispose method.
        // Do not make this method virtual.
        // A derived class should not be able to override this method.
        public void Dispose()
        {
        	Dispose(true);
        	// Take yourself off the Finalization queue
        	// to prevent finalization code for this object
        	// from executing a second time.
        	GC.SuppressFinalize(this);
        }
        
        // Dispose(bool disposing) executes in two distinct scenarios.
        // If disposing equals true, the method has been called directly
        // or indirectly by a user's code. Managed and unmanaged resources
        // can be disposed.
        // If disposing equals false, the method has been called by the
        // runtime from inside the finalizer and you should not reference
        // other objects. Only unmanaged resources can be disposed.
        protected virtual void Dispose(bool disposing)
        {
        	// Check to see if Dispose has already been called.
        	if(!FDisposed)
        	{
        		if(disposing)
        		{
        			// Dispose managed resources.
        		}
        		// Release unmanaged resources. If disposing is false,
        		// only the following code is executed.
	        	
        		FHost.Log(TLogType.Debug, "Bluetooth pluging is being deleted");
        		
        		// Note that this is not thread safe.
        		// Another thread could start disposing the object
        		// after the managed resources are disposed,
        		// but before the disposed flag is set to true.
        		// If thread safety is necessary, it must be
        		// implemented by the client.
        	}
        	FDisposed = true;
        	
        	listener.Close();
        	try {thread.Abort();}
        	catch(Exception e) {}
        	
        }

        // Use C# destructor syntax for finalization code.
        // This destructor will run only if the Dispose method
        // does not get called.
        // It gives your base class the opportunity to finalize.
        // Do not provide destructors in types derived from this class.
        ~PluginTemplate()
        {
        	// Do not re-create Dispose clean-up code here.
        	// Calling Dispose(false) is optimal in terms of
        	// readability and maintainability.
        	Dispose(false);
        }
        #endregion constructor/destructor
        
        #region node name and infos
       
        //provide node infos 
        private static IPluginInfo FPluginInfo;
        public static IPluginInfo PluginInfo
	    {
	        get 
	        {
	        	if (FPluginInfo == null)
				{
					//fill out nodes info
					//see: http://www.vvvv.org/tiki-index.php?page=vvvv+naming+conventions
					FPluginInfo = new PluginInfo();
					
					//the nodes main name: use CamelCaps and no spaces
					FPluginInfo.Name = "Bluetooth Receiver";
					//the nodes category: try to use an existing one
					FPluginInfo.Category = "Network";
					//the nodes version: optional. leave blank if not
					//needed to distinguish two nodes of the same name and category
					FPluginInfo.Version = "OBEX";
					
					//the nodes author: your sign
					FPluginInfo.Author = "Nicolas Baumgardt";
					//describe the nodes function
					FPluginInfo.Help = "Offers basic functionality for sending objects via OBEX protocol to Bluetooth nodes.";
					//specify a comma separated list of tags that describe the node
					FPluginInfo.Tags = "";
					
					//give credits to thirdparty code used
					FPluginInfo.Credits = "32feet.net";
					//any known problems?
					FPluginInfo.Bugs = "";
					//any known usage of the node that may cause troubles?
					FPluginInfo.Warnings = "";
					
					//leave below as is
					System.Diagnostics.StackTrace st = new System.Diagnostics.StackTrace(true);
					System.Diagnostics.StackFrame sf = st.GetFrame(0);
					System.Reflection.MethodBase method = sf.GetMethod();
					FPluginInfo.Namespace = method.DeclaringType.Namespace;
					FPluginInfo.Class = method.DeclaringType.Name;
					//leave above as is
				}
				return FPluginInfo;
	        }
		}

        public bool AutoEvaluate
        {
        	//return true if this node needs to calculate every frame even if nobody asks for its output
        	get {return true;}
        }
        
        #endregion node name and infos
        
      	#region pin creation
        
        //this method is called by vvvv when the node is created
        public void SetPluginHost(IPluginHost Host)
	    {
        	//assign host
	    	FHost = Host;

	    	//create inputs
	    	FHost.CreateStringInput("Save To", TSliceMode.Single, TPinVisibility.True, out dstFPTHStrInp);
	    	dstFPTHStrInp.SetSubType("c:\\", false);	
	    	
	    	FHost.CreateValueInput("Do Listen", 1, null, TSliceMode.Single, TPinVisibility.True, out listenValInp);
	    	listenValInp.SetSubType(0.0, 1.0, 1.0, 0.0, false, true, false);
	    	
	    	//create outputs
	    	FHost.CreateStringOutput("Status", TSliceMode.Single, TPinVisibility.True, out msgStrOut);
	    	msgStrOut.SetSubType("sleeping", false);
	    	
	    	FHost.CreateValueOutput("Received", 1, null, TSliceMode.Single, TPinVisibility.True, out receivedValOut);
	    	receivedValOut.SetSubType(0.0, 1.0, 1.0, 0.0, false, true, false);
	    	
	    	FHost.CreateStringOutput("Filename", TSliceMode.Single, TPinVisibility.True, out fileStrOut);
	    	fileStrOut.SetSubType("", false);
	    	
	    	FHost.CreateStringOutput("Sender Address", TSliceMode.Single, TPinVisibility.True, out macStrOut);
	    	macStrOut.SetSubType("", false);
	    	
        }

        #endregion pin creation
        
        #region mainloop
        
        public void Configurate(IPluginConfig Input)
        {
        	//nothing to configure in this plugin
        	//only used in conjunction with inputs of type cmpdConfigurate
        }
        
        //here we go, thats the method called by vvvv each frame
        //all data handling should be in here
        public void Evaluate(int SpreadMax)
        {   
        	
        	//if the scan switch has been changed
        	if (listenValInp.PinIsChanged) {	
	    		
        		//get the switch state
        		double swt;
        		listenValInp.GetValue(0, out swt);
	    		
	    		//if the scan switch has been turned ON
	    		if(swt==1.0) {
	    			listener = new ObexListener(ObexTransport.Bluetooth);
	    			listener.Start();
	    			thread = new Thread(new ThreadStart(listen));
	    			thread.Start();
                	msgStrOut.SetString(0, "listening");
	    		} else {
	    			listener.Close();
	    			try {thread.Abort();}
	    			catch(Exception e) {}
	    			msgStrOut.SetString(0, "sleeping");
	    		}
	    		
	    	}
        	
        	//update the filepath
        	if (dstFPTHStrInp.PinIsChanged) {
        		dstFPTHStrInp.GetString(0, out path);
        	}
        	
        	//handling output messages thrown by the listener thread
        	if(msg) {
        		msgStrOut.SetString(0, "receiving");
        		fileStrOut.SetString(0, filename);
        		macStrOut.SetString(0, remoteMac);
        		msg=false;
        	}
        	
        	//generate a 1-frame signal when an object was received
        	if(received) {
        		receivedValOut.SetValue(0, 1.0);
        		received=false;
        	} else {
        		receivedValOut.SetValue(0, 0.0);
        	}

        }
        
        private void listen() {
        	while(running) {
    			ObexListenerContext olc = listener.GetContext();
    			ObexListenerRequest olr = olc.Request;
    			remoteMac = olr.RemoteEndPoint.ToString();
    			if(remoteMac.Length>12) {
    				remoteMac=remoteMac.Substring(0, 12);
    			}
    			filename = path + olr.RawUrl.Replace("/","").Replace("%EF%BF%BD","");
	            olr.WriteFile(filename);
	            received=true;
	            msg=true;
        	}
        }
             
        #endregion mainloop  
	}
}
